#pylint: disable=line-too-long,attribute-defined-outside-init
'''
    The various bits of action/logic we can perform are listed as a mixin here
'''
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Version 0.1 by Pat Riehecky <riehecky@fnal.gov> for Scientific Linux

# for python3 compat
from __future__ import unicode_literals
from __future__ import absolute_import

from ..package.models import PackageModel
from ..package import Package

import os
import sys

try:
    import xml.etree.cElementTree as xmletree
except ImportError:  # pragma: no cover
    import xml.etree.ElementTree as xmletree

class CollectionEvents(object):
    '''
        Actions we can perform to set data within the Model
    '''
    def __init__(self):
        '''Nothing to do here'''
        pass

    def __delitem__(self, key):
        '''
           Used for: removing the packages like a dict
        '''
        del self._pkglist[key]

    def __setitem__(self, key, value):
        '''
            While we support some dict like actions,
             we do NOT support collection[key] = value
        '''
        raise NotImplementedError("You cannot set collection packages like a dict, use self.add()")

    def add(self, packageobj, merge=True):
        '''
            So you can do collection.add(pkgobject)
        '''
        if xmletree.iselement(packageobj):
            _tmp = Package()
            _tmp.xmletree = packageobj
            packageobj = _tmp

        if not isinstance(packageobj, PackageModel):
            raise TypeError('I can only add PackageModel type objects')

        if packageobj.filename in self._pkglist:
            if merge:
                if packageobj.name:
                    self._pkglist[packageobj.filename].name = packageobj.name
                if packageobj.epoch:
                    self._pkglist[packageobj.filename].epoch = packageobj.epoch
                if packageobj.version:
                    self._pkglist[packageobj.filename].version = packageobj.version
                if packageobj.release:
                    self._pkglist[packageobj.filename].release = packageobj.release
                if packageobj.arch:
                    self._pkglist[packageobj.filename].arch = packageobj.arch
                if packageobj.srpm:
                    self._pkglist[packageobj.filename].srpm = packageobj.srpm
                if packageobj.src_repo_base:
                    self._pkglist[packageobj.filename].src_repo_base = packageobj.src_repo_base
                if packageobj.builddate:
                    self._pkglist[packageobj.filename].builddate = packageobj.builddate
                if packageobj.reboot_suggested:
                    self._pkglist[packageobj.filename].reboot_suggested = packageobj.reboot_suggested
                if packageobj.restart_suggested:
                    self._pkglist[packageobj.filename].restart_suggested = packageobj.restart_suggested
                if packageobj.relogin_suggested:
                    self._pkglist[packageobj.filename].relogin_suggested = packageobj.relogin_suggested

                for chksum in packageobj.sums:
                    self._pkglist[packageobj.filename].sums[chksum] = packageobj.sums[chksum]

            else:
                raise ValueError('Adding duplicate when merge disabled')

        self._pkglist[packageobj.filename] = packageobj

        if packageobj.filename in self:
            return True

        raise RuntimeError('Attempted to add package to collection, but not found after add')  # pragma: no cover

    def add_filename(self, filename, readfile=False, merge=True, checksum=None):
        '''
            So you can do
              collection.add_filename('my.rpm')
              collection.add_filename('/path/to/my.rpm', readfile=True)
        '''
        newpkg = Package()
        newpkg.filename = filename
        if readfile:
            newpkg.filename = os.path.basename(filename)
            newpkg.set_from_file(filename, checksum=checksum)

        self.add(newpkg, merge=merge)

    def remove(self, packageobj):
        '''
            So you can do collection.remove(pkg)
        '''
        filename = None
        if isinstance(packageobj, PackageModel):
            filename = packageobj.filename
        elif isinstance(packageobj, str):
            filename = packageobj
        elif (sys.version_info < (3, 0)):
            # I'm a hack for py2/3 compat!
            if isinstance(packageobj, unicode):
                filename = packageobj
            else:
                raise ValueError('Pass either the filename, or the package object to remove')
        else:
            raise ValueError('Pass either the filename, or the package object to remove')

        del self[filename]

        if filename not in self:
            return True

        raise RuntimeError('Attempted to remove package from collection, but found after del')  # pragma: no cover

