#pylint: disable=line-too-long
'''
    A set of helper functions for dealing with repodata
'''
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Version 0.1 by Pat Riehecky <riehecky@fnal.gov> for Scientific Linux

# for python3 compat
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import print_function

import os
import sys

try:
    import xml.etree.cElementTree as xmletree
except ImportError:  # pragma: no cover
    import xml.etree.ElementTree as xmletree

def add_xml_to_repo(xmltxt, repobase, filename='updateinfo.xml', sumtype='sha256', cleanup=True):
    '''
        Configures the repomd.xml to know about this new updateinfo.xml

        It will remove the old one automatically if you tell it
         what it was called.
    '''
    # this is where modifyrepo.py is kept on SL6
    sys.path.append('/usr/share/createrepo/')
    try:  # pragma: no cover
        #pylint: disable=import-error
        from modifyrepo import RepoMetadata
    except ImportError as error_msg:
        print(' ', file=sys.stderr)
        print("Could not locate modifyrepo.py", file=sys.stderr)
        print("That is odd... should be with createrepo", file=sys.stderr)
        raise ImportError(error_msg)

    if '~' in repobase:
        repobase = os.path.expanduser(repobase)
    if not repobase.startswith('/'):
        repobase = os.path.abspath(repobase)

    if cleanup:
        if os.path.isfile(repobase + '/repodata/updateinfo.xml'):
            os.remove(repobase + '/repodata/updateinfo.xml')
        if os.path.isfile(repobase + '/repodata/' + filename):
            os.remove(repobase + '/repodata/' + filename)

    _cwd = os.getcwd()
    os.chdir(repobase + '/repodata')

    _fd = open(filename, 'w')
    _fd.write(xmltxt)
    _fd.close()

    repomd = RepoMetadata('.')
    repomd.compress = True
    repomd.compress_type = 'bz2'
    repomd.checksum_type = sumtype
    repomd.unique_md_filenames = False

    # modifyrepo prints to stdout on its own, catching it
    rightout = sys.stdout
    devnull = open(os.devnull, 'w')
    sys.stdout = devnull
    # lame cast as str, cf: RHBZ 1181253
    repomd.add(str(filename), mdtype='updateinfo')
    sys.stdout = rightout
    devnull.close()

    os.chdir(_cwd)

def get_xml_from_repo(repobase, mdtype='updateinfo'):
    '''
        Pass in the repobase and you will get back the text of the
        xml that is tagged as a updateinfo metadata
    '''
    import urllib2
    if repobase.startswith('http://') or repobase.startswith('https://'):
        repomd_xml = urllib2.urlopen(repobase + '/repodata/repomd.xml')

    elif repobase.startswith('ftp://'):
        repomd_xml = urllib2.urlopen(repobase + '/repodata/repomd.xml')

    else:
        repobase = os.path.expanduser(repobase)
        repobase = os.path.abspath(repobase)
        repomd_xml = open(repobase + '/repodata/repomd.xml', 'r')

    repomd = xmletree.fromstring(repomd_xml.read())

    xsd_ns = '{http://linux.duke.edu/metadata/repo}'
    xmlfile = None
    text = None
    for repometadata in repomd:
        if 'type' in repometadata.attrib:
            if repometadata.attrib['type'] == mdtype:
                for inform in repometadata:
                    if inform.tag == xsd_ns + 'location':
                        xmlfile = repobase + '/' + inform.attrib['href']

    if xmlfile.startswith('http://') or xmlfile.startswith('ftp://'):
        repo_xml = urllib2.urlopen(xmlfile)
        if xmlfile.endswith('.gz'):
            import gzip
            raw_xml = gzip.GzipFile(fileobj=repo_xml)
            text = raw_xml.read()
        else:
            text = repo_xml.read()
    else:
        # don't have to fetch local files
        if xmlfile.endswith('.gz'):
            import gzip
            raw_xml = gzip.GzipFile(filename=xmlfile)
            text = raw_xml.read()
        elif xmlfile.endswith('.bz2'):
            import bz2
            raw_xml = bz2.BZ2File(filename=xmlfile)
            text = raw_xml.read()
        elif xmlfile.endswith('.xz'):
            import lzma
            raw_xml = lzma.LZMAFile(filename=xmlfile)
            text = raw_xml.read()
        else:
            _fd = open(xmlfile, 'r')
            text = _fd.read()
            _fd.close()

    return text

def get_package_list_from_repo(repobase):
    '''
        Pass in your repobase and I'll return a list of packages in it
    '''
    primaryxml = get_xml_from_repo(repobase, mdtype='primary')
    primaryxml_obj = xmletree.fromstring(primaryxml)

    common_ns = '{http://linux.duke.edu/metadata/common}'

    rpm_list = set()

    for element in primaryxml_obj:
        for child in element:
            if child.tag == common_ns + 'location':
                rpm_list.add(child.attrib['href'])

    return tuple(rpm_list)

def get_package_list_by_srpm_from_repo(repobase):
    '''
        Pass in your repobase and I'll return a dict of packages
        where the SRPM is the key and packages it builds are the value.
    '''
    primaryxml = get_xml_from_repo(repobase, mdtype='primary')
    primaryxml_obj = xmletree.fromstring(primaryxml)

    common_ns = '{http://linux.duke.edu/metadata/common}'
    rpm_ns = '{http://linux.duke.edu/metadata/rpm}'

    rpm_list = {}

    for element in primaryxml_obj:
        package = element.find("./" + common_ns + 'location').attrib['href']
        sourcerpm = element.find(".//" + rpm_ns + 'sourcerpm')
        if sourcerpm:
            sourcerpm = sourcerpm.text
        else:
            if package.endswith('.src.rpm'):
                sourcerpm = package
            else:
                continue

        if sourcerpm not in rpm_list:
            rpm_list[sourcerpm] = []

        rpm_list[sourcerpm].append(package)

    return rpm_list

def get_package_stanza_from_repo(repobase, packagename):
    '''
        Pass in your repobase and the name of the package as listed in
        <location href="" />
        And I'll return the xmletree of that package's stanza.
    '''
    primaryxml = get_xml_from_repo(repobase, mdtype='primary')
    primaryxml_obj = xmletree.fromstring(primaryxml)

    common_ns = '{http://linux.duke.edu/metadata/common}'

    for element in primaryxml_obj:
        for child in element:
            if child.tag == common_ns + 'location':
                if child.attrib['href'].endswith(os.path.basename(packagename)):
                    return element

    return None

