#pylint: disable=line-too-long
'''
    I'm the unit tests for the helpers
'''
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Version 0.1 by Pat Riehecky <riehecky@fnal.gov> for Scientific Linux

# for python3 compat
from __future__ import unicode_literals
from __future__ import absolute_import

import os
import shutil
import subprocess
import tempfile
import unittest

try:
    import xml.etree.cElementTree as xmletree
except ImportError:  # pragma: no cover
    import xml.etree.ElementTree as xmletree

from .dates import find_old_dates

from .finders import all_updates_for_rpm_name
from .finders import what_update_has_filename
from .finders import what_update_has_reference
from .finders import what_collection_has_filename

from .repo import add_xml_to_repo
from .repo import get_xml_from_repo
from .repo import get_package_list_from_repo
from .repo import get_package_list_by_srpm_from_repo
from .repo import get_package_stanza_from_repo

from .suggested import set_suggested

from .xmltools import xml_pretty_formatter
from .xmltools import get_xsl_pi
from .xmltools import add_xsd_uri
from .xmltools import add_comment
from .xmltools import validate

from ..updateinfo import Updateinfo
from ..update import Update
from ..collection import Collection
from ..reference import Reference
from ..package import Package

class HelperDateTests(unittest.TestCase): # pragma: no cover
    '''Make sure the helper date functions work'''
    Updateinfo = Updateinfo
    Update = Update
    Collection = Collection
    Reference = Reference
    Package = Package
    def pkg_one_for_test(self):
        '''sample'''
        testpkg = self.Package()
        testpkg.name = 'asdf'
        testpkg.version = 'jkl'
        testpkg.release = 'gh'
        testpkg.arch = 'src'
        return testpkg
    def pkg_two_for_test(self):
        '''sample'''
        testpkg = self.Package()
        testpkg.name = 'aaasdf'
        testpkg.version = 'jkl'
        testpkg.release = 'gh'
        testpkg.arch = 'noarch'
        return testpkg
    def coll_one_for_test(self):
        '''sample'''
        testcoll = self.Collection(release_name='jkl', short_name='asdf')
        testcoll.add(self.pkg_one_for_test())
        testcoll.add(self.pkg_two_for_test())
        return testcoll
    def coll_two_for_test(self):
        '''sample'''
        testcoll = self.Collection(release_name='qwe', short_name='iop')
        testcoll.add(self.pkg_one_for_test())
        testcoll.add(self.pkg_two_for_test())
        return testcoll
    def ref_one_for_test(self):
        '''sample'''
        testref = self.Reference(reftype='bugzilla', href='http://1', refid='1', title='asdf')
        return testref
    def ref_two_for_test(self):
        '''sample'''
        testref = self.Reference(reftype='self', href='http://2', refid='2', title='jkl')
        return testref
    def entry_one_for_test(self):
        '''sample'''
        testentry = self.Update()
        testentry.status = 'stable'
        testentry.updatefrom = 'me@example.com'
        testentry.updatetype = 'security'
        testentry.updateid = '1'
        testentry.issued_date = 0
        testentry.collections.add(self.coll_one_for_test())
        testentry.references.add(self.ref_one_for_test())
        return testentry
    def entry_two_for_test(self):
        '''sample'''
        testentry = self.Update()
        testentry.status = 'final'
        testentry.updatefrom = 'me@example.com'
        testentry.updatetype = 'security'
        testentry.updateid = '2'
        testentry.issued_date = '2014-01-01'
        testentry.collections.add(self.coll_two_for_test())
        testentry.references.add(self.ref_two_for_test())
        return testentry
    def updateinfo_for_test(self):
        '''sample'''
        testobj = self.Updateinfo()
        testobj.add(self.entry_one_for_test())
        testobj.add(self.entry_two_for_test())
        return testobj

    def test_old_dates_noarg(self):
        '''do we find super old dates'''
        testobj = self.updateinfo_for_test()

        old = find_old_dates(testobj)

        result = False
        if old == ('1',):
            result = True
        self.assertTrue(result, msg="didn't find old dates")

    def test_old_dates_arg(self):
        '''do we find super old dates'''
        testobj = self.updateinfo_for_test()

        old = find_old_dates(testobj, '1999-10-30')

        result = False
        if old == ('1',):
            result = True
        self.assertTrue(result, msg="didn't find old dates")

class HelperFindersTests(unittest.TestCase): # pragma: no cover
    '''Make sure the helper id functions work'''
    Updateinfo = Updateinfo
    Update = Update
    Collection = Collection
    Reference = Reference
    Package = Package
    def pkg_one_for_test(self):
        '''sample'''
        testpkg = self.Package()
        testpkg.name = 'asdf'
        testpkg.version = 'jkl'
        testpkg.release = 'gh'
        testpkg.arch = 'src'
        return testpkg
    def pkg_two_for_test(self):
        '''sample'''
        testpkg = self.Package()
        testpkg.name = 'aaasdf'
        testpkg.version = 'jkl'
        testpkg.release = 'gh'
        testpkg.arch = 'noarch'
        return testpkg
    def pkg_three_for_test(self):
        '''sample'''
        testpkg = self.Package()
        testpkg.name = 'aaasdf'
        testpkg.version = '2'
        testpkg.release = 'gh'
        testpkg.arch = 'noarch'
        return testpkg
    def pkg_three_again_for_test(self):
        '''sample'''
        testpkg = self.Package()
        testpkg.name = 'xxbxbxbx'
        testpkg.version = '2'
        testpkg.release = 'gh'
        testpkg.arch = 'noarch'
        return testpkg
    def pkg_four_for_test(self):
        '''sample'''
        testpkg = self.Package()
        testpkg.name = 'somename'
        testpkg.version = '2'
        testpkg.release = 'gh'
        testpkg.arch = 'noarch'
        return testpkg
    def coll_one_for_test(self):
        '''sample'''
        testcoll = self.Collection(release_name='jkl', short_name='asdf')
        testcoll.add(self.pkg_one_for_test())
        return testcoll
    def coll_two_for_test(self):
        '''sample'''
        testcoll = self.Collection(release_name='qwe', short_name='iop')
        testcoll.add(self.pkg_two_for_test())
        return testcoll
    def coll_three_for_test(self):
        '''sample'''
        testcoll = self.Collection(release_name='qwe', short_name='iop')
        testcoll.add(self.pkg_three_for_test())
        testcoll.add(self.pkg_three_again_for_test())
        return testcoll
    def coll_three_again_for_test(self):
        '''sample'''
        testcoll = self.Collection(release_name='ebebe', short_name='bbrrbr')
        testcoll.add(self.pkg_three_again_for_test())
        return testcoll
    def coll_four_for_test(self):
        '''sample'''
        testcoll = self.Collection(release_name='qwe', short_name='iop')
        testcoll.add(self.pkg_four_for_test())
        return testcoll
    def ref_one_for_test(self):
        '''sample'''
        testref = self.Reference(reftype='bugzilla', href='http://1', refid='1', title='asdf')
        return testref
    def ref_two_for_test(self):
        '''sample'''
        testref = self.Reference(reftype='self', href='http://2', refid='2', title='jkl')
        return testref
    def entry_one_for_test(self):
        '''sample'''
        testentry = self.Update()
        testentry.status = 'stable'
        testentry.updatefrom = 'me@example.com'
        testentry.updatetype = 'security'
        testentry.updateid = '1'
        testentry.issued_date = 0
        testentry.collections.add(self.coll_one_for_test())
        testentry.references.add(self.ref_one_for_test())
        return testentry
    def entry_two_for_test(self):
        '''sample'''
        testentry = self.Update()
        testentry.status = 'final'
        testentry.updatefrom = 'me@example.com'
        testentry.updatetype = 'security'
        testentry.updateid = '2'
        testentry.issued_date = '2014-01-01'
        testentry.collections.add(self.coll_two_for_test())
        testentry.references.add(self.ref_two_for_test())
        return testentry
    def entry_three_for_test(self):
        '''sample'''
        testentry = self.Update()
        testentry.status = 'final'
        testentry.updatefrom = 'me@example.com'
        testentry.updatetype = 'security'
        testentry.updateid = '3'
        testentry.issued_date = '2014-01-01'
        testentry.collections.add(self.coll_three_for_test())
        return testentry
    def entry_four_for_test(self):
        '''sample'''
        testentry = self.Update()
        testentry.status = 'final'
        testentry.updatefrom = 'me@example.com'
        testentry.updatetype = 'security'
        testentry.updateid = '4'
        testentry.issued_date = '2014-01-01'
        testentry.collections.add(self.coll_four_for_test())
        return testentry
    def updateinfo_for_test(self):
        '''sample'''
        testobj = self.Updateinfo()
        testobj.add(self.entry_one_for_test())
        testobj.add(self.entry_two_for_test())
        testobj.add(self.entry_three_for_test())
        testobj.add(self.entry_four_for_test())
        return testobj

    def test_what_update_has_filename_has(self):
        '''See if files we know are there are where we expect'''
        testobj = self.updateinfo_for_test()

        found = what_update_has_filename(testobj, 'asdf-jkl-gh.src.rpm')

        result = False
        if found == ('1',):
            result = True

        self.assertTrue(result, msg="didn't find package")

    def test_what_update_has_filename_nope(self):
        '''See if files we know are there are where we expect'''
        testobj = self.updateinfo_for_test()

        found = what_update_has_filename(testobj, 'src.rpm')

        result = False
        if found == ():
            result = True

        self.assertTrue(result, msg="found something")

    def test_what_collection_has_filename_has(self):
        '''See if files we know are there are where we expect'''
        testobj = self.updateinfo_for_test()

        found = what_collection_has_filename(testobj['1'], 'asdf-jkl-gh.src.rpm')

        result = False
        if found == ('asdf',):
            result = True

        self.assertTrue(result, msg="didn't find package")

    def test_what_collection_has_filename_nope(self):
        '''See if files we know are there are where we expect'''
        testobj = self.updateinfo_for_test()

        found = what_collection_has_filename(testobj['1'], 'src.rpm')

        result = False
        if found == ():
            result = True

        self.assertTrue(result, msg="found something")

    def test_what_update_has_reference_has(self):
        '''See if files we know are there are where we expect'''
        testobj = self.updateinfo_for_test()

        found = what_update_has_reference(testobj, 'http://2')

        result = False
        if found == ('2',):
            result = True

        self.assertTrue(result, msg="didn't find package")

    def test_what_update_has_reference_nope(self):
        '''See if files we know are there are where we expect'''
        testobj = self.updateinfo_for_test()

        found = what_update_has_reference(testobj, 'http://3')

        result = False
        if found == ():
            result = True

        self.assertTrue(result, msg="found something")

    def test_all_updates_for_rpm_name_has_one(self):
        '''see if we can find just one update'''
        testobj = self.updateinfo_for_test()
        found = all_updates_for_rpm_name(testobj, 'asdf')

        result = False
        if found == ('1',):
            result = True

        self.assertTrue(result, msg="couldn't find one")

    def test_all_updates_for_rpm_name_has_two(self):
        '''see if we can find several updates'''
        testobj = self.updateinfo_for_test()
        found = all_updates_for_rpm_name(testobj, 'aaasdf')

        result = False
        if found == ('2','3'):
            result = True

        self.assertTrue(result, msg="couldn't find multiple")

    def test_all_updates_for_rpm_name_has_none(self):
        '''see if we can find several updates'''
        testobj = self.updateinfo_for_test()
        found = all_updates_for_rpm_name(testobj, 'f')

        result = False
        if found == ():
            result = True

        self.assertTrue(result, msg="found something")

    def test_all_updates_for_rpm_name_none(self):
        '''see if we can find several updates'''
        testobj = self.updateinfo_for_test()
        found = all_updates_for_rpm_name(testobj, None)

        result = False
        if found == ():
            result = True

        self.assertTrue(result, msg="found something")

class HelperRepoTests(unittest.TestCase): # pragma: no cover
    '''Make sure the helper repo functions work'''
    Updateinfo = Updateinfo
    Update = Update
    Collection = Collection
    Reference = Reference
    Package = Package
    def pkg_one_for_test(self):
        '''sample'''
        testpkg = self.Package()
        testpkg.name = 'asdf'
        testpkg.version = 'jkl'
        testpkg.release = 'gh'
        testpkg.arch = 'src'
        return testpkg
    def pkg_two_for_test(self):
        '''sample'''
        testpkg = self.Package()
        testpkg.name = 'aaasdf'
        testpkg.version = 'jkl'
        testpkg.release = 'gh'
        testpkg.arch = 'noarch'
        return testpkg
    def coll_one_for_test(self):
        '''sample'''
        testcoll = self.Collection(release_name='jkl', short_name='asdf')
        testcoll.add(self.pkg_one_for_test())
        testcoll.add(self.pkg_two_for_test())
        return testcoll
    def coll_two_for_test(self):
        '''sample'''
        testcoll = self.Collection(release_name='qwe', short_name='iop')
        testcoll.add(self.pkg_one_for_test())
        testcoll.add(self.pkg_two_for_test())
        return testcoll
    def ref_one_for_test(self):
        '''sample'''
        testref = self.Reference(reftype='bugzilla', href='http://1', refid='1', title='asdf')
        return testref
    def ref_two_for_test(self):
        '''sample'''
        testref = self.Reference(reftype='self', href='http://2', refid='2', title='jkl')
        return testref
    def entry_one_for_test(self):
        '''sample'''
        testentry = self.Update()
        testentry.status = 'stable'
        testentry.updatefrom = 'me@example.com'
        testentry.updatetype = 'security'
        testentry.updateid = '1'
        testentry.issued_date = 0
        testentry.collections.add(self.coll_one_for_test())
        testentry.references.add(self.ref_one_for_test())
        return testentry
    def entry_two_for_test(self):
        '''sample'''
        testentry = self.Update()
        testentry.status = 'final'
        testentry.updatefrom = 'me@example.com'
        testentry.updatetype = 'security'
        testentry.updateid = '2'
        testentry.issued_date = 0
        testentry.collections.add(self.coll_two_for_test())
        testentry.references.add(self.ref_two_for_test())
        return testentry
    def updateinfo_for_test(self):
        '''sample'''
        testobj = self.Updateinfo()
        testobj.add(self.entry_one_for_test())
        testobj.add(self.entry_two_for_test())
        return testobj

    def test_write_to_cwd_repo(self):
        '''try to write to repodata'''
        uinfo = self.updateinfo_for_test()
        tmpdir = tempfile.mkdtemp()

        _cwd = os.getcwd()
        os.chdir(tmpdir)
        subprocess.call(['createrepo', '.'], stdout=subprocess.PIPE)

        add_xml_to_repo(uinfo.xml, '.')

        _fd = open('repodata/repomd.xml', 'r')
        xmlfile = _fd.read()
        _fd.close()

        repomd = xmletree.fromstring(xmlfile)

        xsd_ns = '{http://linux.duke.edu/metadata/repo}'
        uinfofile = None
        text = None
        for repometadata in repomd:
            if 'type' in repometadata.attrib:
                if repometadata.attrib['type'] == 'updateinfo':
                    for inform in repometadata:
                        if inform.tag == xsd_ns + 'location':
                            uinfofile = inform.attrib['href']

        os.chdir(_cwd)
        shutil.rmtree(tmpdir)

        result = False
        if uinfofile == 'repodata/updateinfo.xml.bz2':
            result = True

        self.assertTrue(result, msg='relative write not working')

    def test_write_to_some_repo(self):
        '''try to write to repodata'''
        uinfo = self.updateinfo_for_test()
        tmpdir = tempfile.mkdtemp()

        subprocess.call(['createrepo', tmpdir], stdout=subprocess.PIPE)

        add_xml_to_repo(uinfo.xml, tmpdir, cleanup=False)

        _fd = open(tmpdir + '/repodata/repomd.xml', 'r')
        xmlfile = _fd.read()
        _fd.close()

        repomd = xmletree.fromstring(xmlfile)

        xsd_ns = '{http://linux.duke.edu/metadata/repo}'
        uinfofile = None
        text = None
        for repometadata in repomd:
            if 'type' in repometadata.attrib:
                if repometadata.attrib['type'] == 'updateinfo':
                    for inform in repometadata:
                        if inform.tag == xsd_ns + 'location':
                            uinfofile = inform.attrib['href']

        shutil.rmtree(tmpdir)

        result = False
        if uinfofile == 'repodata/updateinfo.xml.bz2':
            result = True
        self.assertTrue(result, msg='abs write not working')

    def test_write_to_tilde_repo(self):
        '''try to write to repodata'''
        uinfo = self.updateinfo_for_test()
        tmpdir = '~/.__DELETE_ME_some_random_dirname_'
        try:
            shutil.rmtree(os.path.expanduser(tmpdir))
        except:
            pass
        os.mkdir(os.path.expanduser(tmpdir))

        subprocess.call(['createrepo', os.path.expanduser(tmpdir)], stdout=subprocess.PIPE)

        add_xml_to_repo(uinfo.xml, tmpdir)

        _fd = open(os.path.expanduser(tmpdir) + '/repodata/repomd.xml', 'r')
        xmlfile = _fd.read()
        _fd.close()

        repomd = xmletree.fromstring(xmlfile)

        xsd_ns = '{http://linux.duke.edu/metadata/repo}'
        uinfofile = None
        text = None
        for repometadata in repomd:
            if 'type' in repometadata.attrib:
                if repometadata.attrib['type'] == 'updateinfo':
                    for inform in repometadata:
                        if inform.tag == xsd_ns + 'location':
                            uinfofile = inform.attrib['href']

        shutil.rmtree(os.path.expanduser(tmpdir))

        result = False
        if uinfofile == 'repodata/updateinfo.xml.bz2':
            result = True
        self.assertTrue(result, msg='tilde write not working')

    def test_write_to_some_repo_cleanup(self):
        '''try to write to repodata'''
        uinfo = self.updateinfo_for_test()
        tmpdir = tempfile.mkdtemp()

        subprocess.call(['createrepo', tmpdir], stdout=subprocess.PIPE)

        add_xml_to_repo(uinfo.xml, tmpdir)
        add_xml_to_repo(uinfo.xml, tmpdir, cleanup=True)

        _fd = open(tmpdir + '/repodata/repomd.xml', 'r')
        xmlfile = _fd.read()
        _fd.close()

        repomd = xmletree.fromstring(xmlfile)

        xsd_ns = '{http://linux.duke.edu/metadata/repo}'
        uinfofile = None
        text = None
        for repometadata in repomd:
            if 'type' in repometadata.attrib:
                if repometadata.attrib['type'] == 'updateinfo':
                    for inform in repometadata:
                        if inform.tag == xsd_ns + 'location':
                            uinfofile = inform.attrib['href']

        shutil.rmtree(tmpdir)

        result = False
        if uinfofile == 'repodata/updateinfo.xml.bz2':
            result = True
        self.assertTrue(result, msg='abs write not working')

    def test_write_to_some_repo_cleanup_filename(self):
        '''try to write to repodata'''
        uinfo = self.updateinfo_for_test()
        tmpdir = tempfile.mkdtemp()

        subprocess.call(['createrepo', tmpdir], stdout=subprocess.PIPE)

        add_xml_to_repo(uinfo.xml, tmpdir, filename='deleteme')
        add_xml_to_repo(uinfo.xml, tmpdir, filename='deleteme', cleanup=True)

        _fd = open(tmpdir + '/repodata/repomd.xml', 'r')
        xmlfile = _fd.read()
        _fd.close()

        repomd = xmletree.fromstring(xmlfile)

        xsd_ns = '{http://linux.duke.edu/metadata/repo}'
        uinfofile = None
        text = None
        for repometadata in repomd:
            if 'type' in repometadata.attrib:
                if repometadata.attrib['type'] == 'updateinfo':
                    for inform in repometadata:
                        if inform.tag == xsd_ns + 'location':
                            uinfofile = inform.attrib['href']

        shutil.rmtree(tmpdir)

        result = False
        if uinfofile == 'repodata/deleteme.bz2':
            result = True

        self.assertTrue(result, msg='abs write not working')

    def test_get_from_cwd_repo(self):
        '''try to read from repodata'''
        uinfo = self.updateinfo_for_test()
        tmpdir = tempfile.mkdtemp()

        subprocess.call(['createrepo', tmpdir], stdout=subprocess.PIPE)

        add_xml_to_repo(uinfo.xml, tmpdir)

        _cwd = os.getcwd()
        os.chdir(tmpdir)

        txt = get_xml_from_repo('.')

        os.chdir(_cwd)
        shutil.rmtree(tmpdir)

        expected = '''<updates><update from="me@example.com" status="stable" type="security" version="1.4"><id>1</id><issued date="1969-12-31 18:00:00" /><pkglist><collection short="asdf"><name>jkl</name><package arch="noarch" name="aaasdf" release="gh" version="jkl"><filename>aaasdf-jkl-gh.noarch.rpm</filename></package><package arch="src" name="asdf" release="gh" version="jkl"><filename>asdf-jkl-gh.src.rpm</filename></package></collection></pkglist><references><reference href="http://1" id="1" title="Asdf" type="bugzilla" /></references></update><update from="me@example.com" status="final" type="security" version="1.4"><id>2</id><issued date="1969-12-31 18:00:00" /><pkglist><collection short="iop"><name>qwe</name><package arch="noarch" name="aaasdf" release="gh" version="jkl"><filename>aaasdf-jkl-gh.noarch.rpm</filename></package><package arch="src" name="asdf" release="gh" version="jkl"><filename>asdf-jkl-gh.src.rpm</filename></package></collection></pkglist><references><reference href="http://2" id="2" title="Jkl" type="self" /></references></update></updates>'''
        result = False
        if txt == expected:
            result = True
        self.assertTrue(result, msg='relative read not working')

    def test_get_from_abs_repo(self):
        '''try to read from repodata'''
        uinfo = self.updateinfo_for_test()
        tmpdir = tempfile.mkdtemp()

        subprocess.call(['createrepo', tmpdir], stdout=subprocess.PIPE)

        add_xml_to_repo(uinfo.xml, tmpdir)

        txt = get_xml_from_repo(tmpdir)

        shutil.rmtree(tmpdir)

        expected = '''<updates><update from="me@example.com" status="stable" type="security" version="1.4"><id>1</id><issued date="1969-12-31 18:00:00" /><pkglist><collection short="asdf"><name>jkl</name><package arch="noarch" name="aaasdf" release="gh" version="jkl"><filename>aaasdf-jkl-gh.noarch.rpm</filename></package><package arch="src" name="asdf" release="gh" version="jkl"><filename>asdf-jkl-gh.src.rpm</filename></package></collection></pkglist><references><reference href="http://1" id="1" title="Asdf" type="bugzilla" /></references></update><update from="me@example.com" status="final" type="security" version="1.4"><id>2</id><issued date="1969-12-31 18:00:00" /><pkglist><collection short="iop"><name>qwe</name><package arch="noarch" name="aaasdf" release="gh" version="jkl"><filename>aaasdf-jkl-gh.noarch.rpm</filename></package><package arch="src" name="asdf" release="gh" version="jkl"><filename>asdf-jkl-gh.src.rpm</filename></package></collection></pkglist><references><reference href="http://2" id="2" title="Jkl" type="self" /></references></update></updates>'''
        result = False
        if txt == expected:
            result = True
        self.assertTrue(result, msg='abs read not working')

    def test_get_packages_from_repo(self):
        '''try to read from repodata'''
        tmpdir = tempfile.mkdtemp()
        subprocess.call(['createrepo', tmpdir], stdout=subprocess.PIPE)

        pkgs = get_package_list_from_repo(tmpdir)
        shutil.rmtree(tmpdir)

        result = False
        if pkgs == ():
            result = True
        self.assertTrue(result, msg="couldn't read pkg metadata")

    def test_get_package_list_by_srpm_from_repo(self):
        '''try to read from repodata'''
        tmpdir = tempfile.mkdtemp()
        subprocess.call(['createrepo', tmpdir], stdout=subprocess.PIPE)

        pkgs = get_package_list_by_srpm_from_repo(tmpdir)
        shutil.rmtree(tmpdir)

        result = False
        if pkgs == {}:
            result = True
        self.assertTrue(result, msg="couldn't read pkg metadata")

    def test_get_package_stanza_from_repo(self):
        '''try to read from repodata'''
        tmpdir = tempfile.mkdtemp()
        subprocess.call(['createrepo', tmpdir], stdout=subprocess.PIPE)

        pkgs = get_package_stanza_from_repo(tmpdir, 'nofile')
        shutil.rmtree(tmpdir)

        result = False
        if pkgs == None:
            result = True
        self.assertTrue(result, msg="couldn't read pkg metadata")

class HelperSuggestedTests(unittest.TestCase): # pragma: no cover
    '''Make sure suggested, suggests'''
    Updateinfo = Updateinfo
    Update = Update
    Collection = Collection
    Reference = Reference
    Package = Package
    def pkg_one_for_test(self):
        '''sample'''
        testpkg = self.Package()
        testpkg.name = 'asdf'
        testpkg.version = 'jkl'
        testpkg.release = 'gh'
        testpkg.arch = 'src'
        return testpkg
    def pkg_two_for_test(self):
        '''sample'''
        testpkg = self.Package()
        testpkg.name = 'aaasdf'
        testpkg.version = 'jkl'
        testpkg.release = 'gh'
        testpkg.arch = 'noarch'
        return testpkg
    def coll_one_for_test(self):
        '''sample'''
        testcoll = self.Collection(release_name='jkl', short_name='asdf')
        testcoll.add(self.pkg_one_for_test())
        testcoll.add(self.pkg_two_for_test())
        return testcoll
    def coll_two_for_test(self):
        '''sample'''
        testcoll = self.Collection(release_name='qwe', short_name='iop')
        testcoll.add(self.pkg_one_for_test())
        testcoll.add(self.pkg_two_for_test())
        return testcoll
    def ref_one_for_test(self):
        '''sample'''
        testref = self.Reference(reftype='bugzilla', href='http://1', refid='1', title='asdf')
        return testref
    def ref_two_for_test(self):
        '''sample'''
        testref = self.Reference(reftype='self', href='http://2', refid='2', title='jkl')
        return testref
    def entry_one_for_test(self):
        '''sample'''
        testentry = self.Update()
        testentry.status = 'stable'
        testentry.updatefrom = 'me@example.com'
        testentry.updatetype = 'security'
        testentry.updateid = '1'
        testentry.title = 'asdf'
        testentry.issued_date = 0
        testentry.collections.add(self.coll_one_for_test())
        testentry.references.add(self.ref_one_for_test())
        return testentry
    def entry_two_for_test(self):
        '''sample'''
        testentry = self.Update()
        testentry.status = 'final'
        testentry.updatefrom = 'me@example.com'
        testentry.updatetype = 'security'
        testentry.updateid = '2'
        testentry.title = 'jkl'
        testentry.issued_date = 0
        testentry.collections.add(self.coll_two_for_test())
        testentry.references.add(self.ref_two_for_test())
        return testentry
    def updateinfo_for_test(self):
        '''sample'''
        testobj = self.Updateinfo()
        testobj.add(self.entry_one_for_test())
        testobj.add(self.entry_two_for_test())
        return testobj

    def test_desktop_restarted(self):
        '''desktop must be restarted'''
        testobj = self.updateinfo_for_test()
        testobj['1'].description = 'the desktop must be restarted'
        set_suggested(testobj['1'])

        self.assertTrue(testobj['1'].relogin_suggested, msg='did not suggest relogin')

    def test_service_restarted(self):
        '''must be restarted'''
        testobj = self.updateinfo_for_test()
        testobj['1'].description = 'the service must be restarted'
        set_suggested(testobj['1'])

        self.assertTrue(testobj['1'].restart_suggested, msg='did not suggest restart')

    def test_system_reboot(self):
        '''must be rebooted'''
        testobj = self.updateinfo_for_test()
        testobj['1'].description = 'your system must be rebooted'
        set_suggested(testobj['1'])
        
        self.assertTrue(testobj['1'].reboot_suggested, msg='did not suggest reboot')

    def test_system_reboot_other(self):
        '''must be rebooted'''
        testobj = self.updateinfo_for_test()
        testobj['1'].description = 'your system rebooted'
        set_suggested(testobj['1'])

        self.assertTrue(testobj['1'].reboot_suggested, msg='did not suggest reboot')


class HelperXMLToolsTests(unittest.TestCase): # pragma: no cover
    '''Make sure the xml helpers work'''
    Updateinfo = Updateinfo
    Update = Update
    Collection = Collection
    Reference = Reference
    Package = Package
    def pkg_one_for_test(self):
        '''sample'''
        testpkg = self.Package()
        testpkg.name = 'asdf'
        testpkg.version = 'jkl'
        testpkg.release = 'gh'
        testpkg.arch = 'src'
        return testpkg
    def pkg_two_for_test(self):
        '''sample'''
        testpkg = self.Package()
        testpkg.name = 'aaasdf'
        testpkg.version = 'jkl'
        testpkg.release = 'gh'
        testpkg.arch = 'noarch'
        return testpkg
    def coll_one_for_test(self):
        '''sample'''
        testcoll = self.Collection(release_name='jkl', short_name='asdf')
        testcoll.add(self.pkg_one_for_test())
        testcoll.add(self.pkg_two_for_test())
        return testcoll
    def coll_two_for_test(self):
        '''sample'''
        testcoll = self.Collection(release_name='qwe', short_name='iop')
        testcoll.add(self.pkg_one_for_test())
        testcoll.add(self.pkg_two_for_test())
        return testcoll
    def ref_one_for_test(self):
        '''sample'''
        testref = self.Reference(reftype='bugzilla', href='http://1', refid='1', title='asdf')
        return testref
    def ref_two_for_test(self):
        '''sample'''
        testref = self.Reference(reftype='self', href='http://2', refid='2', title='jkl')
        return testref
    def entry_one_for_test(self):
        '''sample'''
        testentry = self.Update()
        testentry.status = 'stable'
        testentry.updatefrom = 'me@example.com'
        testentry.updatetype = 'security'
        testentry.updateid = '1'
        testentry.issued_date = 0
        testentry.reboot_suggested = True
        testentry.collections.add(self.coll_one_for_test())
        testentry.references.add(self.ref_one_for_test())
        return testentry
    def entry_two_for_test(self):
        '''sample'''
        testentry = self.Update()
        testentry.status = 'final'
        testentry.updatefrom = 'me@example.com'
        testentry.updatetype = 'security'
        testentry.updateid = '2'
        testentry.issued_date = 0
        testentry.collections.add(self.coll_two_for_test())
        testentry.references.add(self.ref_two_for_test())
        return testentry
    def updateinfo_for_test(self):
        '''sample'''
        testobj = self.Updateinfo()
        testobj.add(self.entry_one_for_test())
        testobj.add(self.entry_two_for_test())
        return testobj

    def test_xml_pretty_formatter_ones(self):
        '''does pprint work?'''
        uinfo = self.updateinfo_for_test()

        mytree = uinfo.xmletree
        xml_pretty_formatter(mytree, _ws=' ')

        expected ='''<updates>
 <update from="me@example.com" status="stable" type="security" version="1.4">
  <id>1</id>
  <issued date="1969-12-31 18:00:00" />
  <reboot_suggested>true</reboot_suggested>
  <pkglist>
   <collection short="asdf">
    <name>jkl</name>
    <package arch="noarch" name="aaasdf" release="gh" version="jkl">
     <filename>aaasdf-jkl-gh.noarch.rpm</filename>
    </package>
    <package arch="src" name="asdf" release="gh" version="jkl">
     <filename>asdf-jkl-gh.src.rpm</filename>
    </package>
   </collection>
  </pkglist>
  <references>
   <reference href="http://1" id="1" title="Asdf" type="bugzilla" />
  </references>
 </update>
 <update from="me@example.com" status="final" type="security" version="1.4">
  <id>2</id>
  <issued date="1969-12-31 18:00:00" />
  <pkglist>
   <collection short="iop">
    <name>qwe</name>
    <package arch="noarch" name="aaasdf" release="gh" version="jkl">
     <filename>aaasdf-jkl-gh.noarch.rpm</filename>
    </package>
    <package arch="src" name="asdf" release="gh" version="jkl">
     <filename>asdf-jkl-gh.src.rpm</filename>
    </package>
   </collection>
  </pkglist>
  <references>
   <reference href="http://2" id="2" title="Jkl" type="self" />
  </references>
 </update>
</updates>
'''
        got = xmletree.tostring(mytree)
        result = False
        if got == expected:
            result = True
        self.assertTrue(result, msg="pretty print not working")

    def test_xml_pretty_formatter_lots(self):
        '''is the whitespace programable?'''
        uinfo = self.updateinfo_for_test()

        mytree = uinfo.xmletree
        xml_pretty_formatter(mytree, _ws='     ')

        expected ='''<updates>
     <update from="me@example.com" status="stable" type="security" version="1.4">
          <id>1</id>
          <issued date="1969-12-31 18:00:00" />
          <reboot_suggested>true</reboot_suggested>
          <pkglist>
               <collection short="asdf">
                    <name>jkl</name>
                    <package arch="noarch" name="aaasdf" release="gh" version="jkl">
                         <filename>aaasdf-jkl-gh.noarch.rpm</filename>
                    </package>
                    <package arch="src" name="asdf" release="gh" version="jkl">
                         <filename>asdf-jkl-gh.src.rpm</filename>
                    </package>
               </collection>
          </pkglist>
          <references>
               <reference href="http://1" id="1" title="Asdf" type="bugzilla" />
          </references>
     </update>
     <update from="me@example.com" status="final" type="security" version="1.4">
          <id>2</id>
          <issued date="1969-12-31 18:00:00" />
          <pkglist>
               <collection short="iop">
                    <name>qwe</name>
                    <package arch="noarch" name="aaasdf" release="gh" version="jkl">
                         <filename>aaasdf-jkl-gh.noarch.rpm</filename>
                    </package>
                    <package arch="src" name="asdf" release="gh" version="jkl">
                         <filename>asdf-jkl-gh.src.rpm</filename>
                    </package>
               </collection>
          </pkglist>
          <references>
               <reference href="http://2" id="2" title="Jkl" type="self" />
          </references>
     </update>
</updates>
'''
        got = xmletree.tostring(mytree)
        result = False
        if got == expected:
            result = True
        self.assertTrue(result, msg="pretty print not working")

    def test_xsl_uri(self):
        '''can I add xsl uris?'''
        xsl_pi = get_xsl_pi('test')
        txt = '''<?xml-stylesheet type="text/xsl" href="test"?>'''

        result = False
        if xmletree.tostring(xsl_pi) == txt:
            result = True
        self.assertTrue(result, msg="Can't get xsluri")

    def test_xsd_uri(self):
        '''can I add xsd uris?'''
        uinfo = self.updateinfo_for_test()

        tree = uinfo.xmletree

        tree = add_xsd_uri('test', tree)

        result = False
        if tree.attrib['xsi:schemaLocation'] == 'test':
            result = True
        self.assertTrue(result, msg="can't find xsd uri, couldn't set it")

    def test_xsl_uri_none(self):
        '''can I add xsl uris?'''

        result = False
        try:
            xsl_pi = get_xsl_pi(None)
        except ValueError:
            result = True
        self.assertTrue(result, msg="Can set xsluri to None")

    def test_xsd_uri_none(self):
        '''can I add xsd uris?'''
        result = False
        try:
            uinfo = self.updateinfo_for_test()
            tree = uinfo.xmletree
            tree = add_xsd_uri(None, tree)
        except ValueError:
            result = True
        self.assertTrue(result, msg="can set xsd uri to None")

    def test_add_xml_comment(self):
        '''can I add xml comments?'''
        uinfo = self.updateinfo_for_test()

        tree = uinfo.xmletree

        tree = add_comment('test', tree)

        txt =  xmletree.tostring(tree).rstrip()
        expected = '''<updates><!--test--><update from="me@example.com" status="stable" type="security" version="1.4"><id>1</id><issued date="1969-12-31 18:00:00" /><reboot_suggested>true</reboot_suggested><pkglist><collection short="asdf"><name>jkl</name><package arch="noarch" name="aaasdf" release="gh" version="jkl"><filename>aaasdf-jkl-gh.noarch.rpm</filename></package><package arch="src" name="asdf" release="gh" version="jkl"><filename>asdf-jkl-gh.src.rpm</filename></package></collection></pkglist><references><reference href="http://1" id="1" title="Asdf" type="bugzilla" /></references></update><update from="me@example.com" status="final" type="security" version="1.4"><id>2</id><issued date="1969-12-31 18:00:00" /><pkglist><collection short="iop"><name>qwe</name><package arch="noarch" name="aaasdf" release="gh" version="jkl"><filename>aaasdf-jkl-gh.noarch.rpm</filename></package><package arch="src" name="asdf" release="gh" version="jkl"><filename>asdf-jkl-gh.src.rpm</filename></package></collection></pkglist><references><reference href="http://2" id="2" title="Jkl" type="self" /></references></update></updates>'''

        ok_too = '''<updates><!-- test --><update from="me@example.com" status="stable" type="security" version="1.4"><id>1</id><issued date="1969-12-31 18:00:00" /><reboot_suggested>true</reboot_suggested><pkglist><collection short="asdf"><name>jkl</name><package arch="noarch" name="aaasdf" release="gh" version="jkl"><filename>aaasdf-jkl-gh.noarch.rpm</filename></package><package arch="src" name="asdf" release="gh" version="jkl"><filename>asdf-jkl-gh.src.rpm</filename></package></collection></pkglist><references><reference href="http://1" id="1" title="Asdf" type="bugzilla" /></references></update><update from="me@example.com" status="final" type="security" version="1.4"><id>2</id><issued date="1969-12-31 18:00:00" /><pkglist><collection short="iop"><name>qwe</name><package arch="noarch" name="aaasdf" release="gh" version="jkl"><filename>aaasdf-jkl-gh.noarch.rpm</filename></package><package arch="src" name="asdf" release="gh" version="jkl"><filename>asdf-jkl-gh.src.rpm</filename></package></collection></pkglist><references><reference href="http://2" id="2" title="Jkl" type="self" /></references></update></updates>'''
        result = False
        if txt == expected or txt == ok_too:
            result = True
        self.assertTrue(result, msg="can't add comments")

    def test_add_xml_comment_none(self):
        '''can I add xml comments?'''
        result = False
        try:
            uinfo = self.updateinfo_for_test()
            tree = uinfo.xmletree
            tree = add_comment(None, tree)
        except ValueError:
            result = True
        self.assertTrue(result, msg="can add comments None comment")

    def test_xsd_works_preformed(self):
        '''Does the XML validate?'''
        from lxml import etree as lxmletree
        txt = '''<updates xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="test"><update from="me@example.com" status="stable" type="security" version="1.4"><id>1</id><issued date="1969-12-31 18:00:00" /><reboot_suggested>true</reboot_suggested><pkglist><collection short="asdf"><name>jkl</name><package arch="noarch" name="aaasdf" release="gh" version="jkl"><filename>aaasdf-jkl-gh.noarch.rpm</filename></package><package epoch="1" arch="src" name="asdf" release="gh" version="jkl"><filename>asdf-jkl-gh.src.rpm</filename></package></collection></pkglist><references><reference href="http://1" id="1" title="Asdf" type="bugzilla" /></references></update></updates>'''

        result = False
        try:
            validate(txt, './docs/updateinfo.xsd')
            result = True
        except lxmletree.DocumentInvalid:
            pass
        self.assertTrue(result, msg="can't validate")

    def test_xsd_works_uinfo_sample_fedora(self):
        '''Does the sample XML validate?'''
        from lxml import etree as lxmletree

        _fd = open('./docs/samples/Fedora-updateinfo.xml', 'r')
        txt = _fd.read()
        _fd.close()

        result = False
        validate(txt, './docs/updateinfo.xsd')
        result = True
        self.assertTrue(result, msg="can't validate")

    def test_xsd_works_uinfo_sample_epel(self):
        '''Does the sample XML validate?'''
        from lxml import etree as lxmletree

        _fd = open('./docs/samples/EPEL6-updateinfo.xml', 'r')
        txt = _fd.read()
        _fd.close()

        result = False
        validate(txt, './docs/updateinfo.xsd')
        result = True
        self.assertTrue(result, msg="can't validate")

    def test_xsd_works_uinfo_made(self):
        '''Does the sample XML validate?'''
        from lxml import etree as lxmletree

        uinfo = self.updateinfo_for_test()

        result = False
        try:
            validate(uinfo.xml, './docs/updateinfo.xsd')
            result = True
        except lxmletree.DocumentInvalid:
            pass
        self.assertTrue(result, msg="can't validate")

