#!/usr/bin/env python
#pylint: disable=line-too-long
'''
    Read in the repodata from an existing yum repo and generate an updateinfo
    for that repo based on an existing updateinfo.xml file.

    Basically this extracts the information based on the filename in <package>,
    but generates new information for the rpms within the repo.
'''
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Version 0.1 by Pat Riehecky <riehecky@fnal.gov> for Scientific Linux

# for python3 compat
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import print_function

import ConfigParser
import os

try:
    import xml.etree.cElementTree as xmletree
except ImportError:
    import xml.etree.ElementTree as xmletree

from updateinfo import Updateinfo

from updateinfo.helpers.finders import what_update_has_filename
from updateinfo.helpers.finders import what_collection_has_filename

from updateinfo.helpers.repo import get_package_list_from_repo
from updateinfo.helpers.repo import get_xml_from_repo
from updateinfo.helpers.repo import add_xml_to_repo

from updateinfo.helpers.xmltools import add_comment
from updateinfo.helpers.xmltools import xml_pretty_formatter

if __name__ == '__main__':
    import textwrap
    from argparse import ArgumentParser

    # Allowing these to be set from either a config or commandline
    # requires some additional magic
    DEFAULTS = {
        'sourcexml': None,
        'repobase': None,
        'collection': 'MyCollection',
        'updatefrom': 'me@example.com',
        'updatemd': False,
        'xsluri': None,
        'quiet': False,
        'readold': True,
        'pretty': False,
        'print': False,
        'commentfile': None,
    }

    CONFPARSER = ArgumentParser(add_help=False)
    CONFPARSER.add_argument('--configfile', metavar='CONFIG.INI',
                            help="Set options from config file (in [DEFAULT])")

    ARGS, REMAINING_ARGS = CONFPARSER.parse_known_args()
    if ARGS.configfile:
        CFG = ConfigParser.SafeConfigParser()
        CFG.read(ARGS.configfile)
        DEFAULTS = dict(list(DEFAULTS.items()) + list(dict(CFG.items('DEFAULT')).items()))

    PARSER = ArgumentParser(description=textwrap.dedent(__doc__),
                            parents=[CONFPARSER])

    PARSER.add_argument('--sourcexml',
                        help='Where is the source updateinfo.xml')
    PARSER.add_argument('--repobase',
                        help='Where is the yum repo we are looking at?')
    PARSER.add_argument('--commentfile', type=file, dest='comment',
                        metavar='/PATH/TO/FILE',
                        help="Add this file's content as an XML comment")
    PARSER.add_argument('--updatemd', action='store_true',
                        help="Should I update that repodata when I'm done?")
    PARSER.add_argument('--xsluri',
                        help='Should I add an XSL uri, to where?')
    PARSER.add_argument('--quiet', action='store_true',
                        help="Should I print out status information?")
    PARSER.add_argument('--no-import', action='store_false', dest='readold',
                        help="Skip importing any existing updateinfo")
    PARSER.add_argument('--pretty', action='store_true',
                        help='Should the output be pretty')
    PARSER.add_argument('--print', action='store_true',
                        help='Should the output be printed to stdout')
    #pylint: disable=star-args
    PARSER.set_defaults(**DEFAULTS)

    ARGS = PARSER.parse_args(REMAINING_ARGS)

    if not (ARGS.sourcexml or ARGS.repobase):
        PARSER.error('You must set sourcexml and repobase')

    MYUPDATEINFO = Updateinfo()

    if ARGS.readold:
        MYUPDATEINFO.xml = get_xml_from_repo(ARGS.repobase)

    SOURCE = Updateinfo()
    _FD = open(os.path.expanduser(ARGS.sourcexml), 'r')
    SOURCE.xml = _FD.read()
    _FD.close()

    PKGS = get_package_list_from_repo(ARGS.repobase)
    for PACKAGENAME in PKGS:
        if PACKAGENAME in MYUPDATEINFO.packages:
            continue
        THISID = what_update_has_filename(SOURCE, PACKAGENAME)
        if not THISID:
            continue

        THISID = THISID[0]
        if THISID not in MYUPDATEINFO:
            MYUPDATEINFO.create(SOURCE[THISID].updateid,
                                SOURCE[THISID].updatefrom,
                                SOURCE[THISID].status,
                                SOURCE[THISID].updatetype,
                                SOURCE[THISID].title,
                                SOURCE[THISID].issued_date,
                                SOURCE[THISID].description,
                                SOURCE[THISID].severity,
                                SOURCE[THISID].releasetitle,
                                SOURCE[THISID].summary,
                                SOURCE[THISID].rights,
                                SOURCE[THISID].solution,
                                SOURCE[THISID].update_date,
                                SOURCE[THISID].reboot_suggested,
                                SOURCE[THISID].restart_suggested,
                                SOURCE[THISID].relogin_suggested)

        THISCOLL = what_collection_has_filename(SOURCE[THISID], PACKAGENAME)[0]
        if THISCOLL not in MYUPDATEINFO[THISID].collections:
            MYUPDATEINFO[THISID].collections.create(SOURCE[THISID].collections[THISCOLL].release_name, SOURCE[THISID].collections[THISCOLL].short_name)

        MYUPDATEINFO[THISID].collections[THISCOLL].add_filename(ARGS.repobase + '/' + PACKAGENAME, readfile=True)

        MYUPDATEINFO[THISID].references = SOURCE[THISID].references

    if not ARGS.quiet:
        NOTFOUND = list(set(PKGS) - set(MYUPDATEINFO.packages))
        NOTFOUND.sort()
        for PKG in NOTFOUND:
            print(PKG)

    XMLOBJ = MYUPDATEINFO.xmletree

    if ARGS.comment:
        XMLOBJ = add_comment(ARGS.comment, XMLOBJ)
    if ARGS.pretty:
        xml_pretty_formatter(XMLOBJ)

    XMLTXT = xmletree.tostring(XMLOBJ)
    if ARGS.xsluri:
        XSLURI = xmletree.tostring(get_xsl_pi(ARGS.xsluri))
        XMLTXT = XSLURI + '\n' + XMLTXT

    if ARGS.updatemd:
        add_xml_to_repo(XMLTXT, ARGS.repobase)

    if ARGS.print:
        print(XMLTXT)

